//===========================================================================
//  @(#) $Name: arts++-1-1-a13 $
//  @(#) $Id: artsagg.cc,v 1.3 2008/04/14 21:17:07 rkoga Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include "artslocal.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>
}

#ifdef HAVE_FSTREAM
  #include <fstream>
#else
  #include <fstream.h>
#endif
#include <string>
#include <cstring> // strlen()
#include <map>

#include "ArtsFileUtil.hh"
#include "ArtsSelectionSet.hh"
#include "ArtsPackageVersion.hh"
#include "ArtsDebug.hh"

extern int   errno;

using namespace std;

const string rcsid = "@(#) $Name: arts++-1-1-a13 $ $Id: artsagg.cc,v 1.3 2008/04/14 21:17:07 rkoga Exp $";
static const ArtsPackageVersion  k_artsVersion = ArtsPackageVersion(rcsid);

//-------------------------------------------------------------------------
//    void Usage(const char *argv0)                      
//-------------------------------------------------------------------------
void Usage(const char *argv0)
{
  cerr << "Usage: " << argv0
       << " [-v] [-i ifIndexList] [-I ifIndexList] [-l numTopPorts]" << endl
       << setw(strlen(argv0) + strlen("Usage: ")) << " "
       << " [-s portlist] [-T objectTypes] [-h hours] [-o] [-q]" << endl
       << setw(strlen(argv0) + strlen("Usage: ")) << " "
       << " outfile infile(s)"
       << endl
       << " - ifIndexList is a comma separated list of interface indices"
       << " or index ranges" << endl
       << "   Example: '1,3,5-12'" << endl
       << " - objectTypes is a comma separated list of object types"
       << " (proto, portm," << endl
       << "   asm, netm, tos, nexthop, or intfm)" << endl
       << "   Example: 'asm,proto,intfm'"
       << endl;
  return;
}

//-------------------------------------------------------------------------
//    int main(int argc, char *argv[])                    
//-------------------------------------------------------------------------
int main(int argc, char *argv[])
{
  extern char     *optarg;
  extern int       optind;
  int              optChar;
  bool             quiet = false;
  bool             overwrite = false;
  float            hours = -1.0;
  ArtsFileUtil    *artsFileUtil = new ArtsFileUtil();
  
  while ((optChar = getopt(argc,argv,"I:h:i:l:oqs:T:v")) != EOF) {
    switch (optChar) {
      case 'I':
        artsFileUtil->IfIndexCombineSet(optarg);
        break;
      case 'h':
        hours = atof(optarg);
        break;
      case 'i':
        artsFileUtil->IfIndexSelectionSet(optarg);
        break;
      case 'l':
        artsFileUtil->NumberOfTopEntries(atoi(optarg));
        break;
      case 'o':
        overwrite = true;
        break;
      case 'q':
        quiet = true;
        break;
      case 's':
        artsFileUtil->PortChooser(optarg);
        break;
      case 'T':
        artsFileUtil->ObjectTypeSelectionSet(optarg);
        break;
      case 'v':
        cerr << k_artsVersion.Name() << "  " << k_artsVersion.Id() << endl;
        exit(0);
        break;
      case '?':
      default:
        Usage(argv[0]);
        exit(1);
        break;
    }
  }

  if (optind >= (argc-1)) {
    Usage(argv[0]);
    exit(1);
  }

  vector<string>  inFilenames;
  
  for (int fileNum = optind+1; fileNum < argc; fileNum++) {
    inFilenames.push_back(string(argv[fileNum]));
  }
  
  artsFileUtil->Aggregate(argv[optind],inFilenames,hours,overwrite,quiet);

  delete(artsFileUtil);
  
  #ifndef NDEBUG
    atexit(ShowArtsObjectsNotFreed);
  #endif
    
  exit(0);
}
