.\" @(#) $Name: arts++-1-1-a13 $ $Id: artsagg.man,v 1.1.1.1 2002/11/16 00:46:38 rkoga Exp $
.TH artsagg l LOCAL
.SH NAME
artsagg \- utility to aggregate ARTS objects in time domain
.SH SYNOPSIS
.nf
artsagg [-v] [-i ifIndexList] [-I ifIndexList] [-l numTopPorts]
        [-s portlist] [-T objectTypes] [-h hours] [-o] [-q] 
        outfile infile(s)
.fi
.SH DESCRIPTION
.I artsagg
is a utility that reads ARTS objects from ARTS files (\fIinfile(s)\fR)
and aggregates them in the time domain, placing the resulting object(s)
into an output file (\fIoutfile\fR).  For example, you may use
\fIartsagg\fR to aggregate 5-minute ARTS data to 1-hour ARTS data.
\fIartsagg\fR aggregates the following ARTS data types:
.nf

  protocol table
  interface matrix
  IP nexthop table
  AS matrix
  net matrix
  port matrix
  TOS table

.fi

.SH OPTIONS
.IP "\fB-v"
The \fI-v\fR option causes \fIartsagg\fR to print version information
and exit immediately.
.IP "\fB-T objectTypes"
The \fI-T\fR option may be used to specify the types of data to be
aggregated.  \fIobjectTypes\fR is a comma-separated list of keywords
for particular data types.  You may use the following keywords:
.nf

  proto         - protocol table
  asm           - AS matrix
  netm          - net matrix
  portm         - port matrix
  tos           - TOS table
  nexthop       - IP nexthop table
  intfm         - interface matrix
.fi
.IP "\fB-i ifIndexList"
The \fI-i ifIndexList\fR option may be used to select only data which
contains an interface index attribute in \fIifIndexList\fR.  In other
words, the \fI-i ifIndexList\fR option lets you select data from
particular interfaces or sets of interfaces.  \fIifIndexList\fR may
contain hyphens (indicating an inclusive range) and commas.  For
example, \fI-i 1,2,5-8\fR would select data with an interface index
attribute of 1, 2, 5, 6, 7 or 8.
.IP "\fB-I ifIndexList"
The \fI-I ifIndexList\fR option may be used to combine data which
contains an interface index attribute in \fIifIndexList\fR.  In other
words, the \fI-I ifIndexList\fR option lets you combine data across a
set of interfaces.  \fIifIndexList\fR may contain hyphens (indicating an
inclusive range) and commas.  For example, \fI-I 1,2,5-8\fR would
combine all data with an interface index attribute of 1, 2, 5, 6,
7 or 8.  The resulting aggregate objects will contain an interface index
attribute of 0.
.IP "\fB-l numTopPorts"
The \fI-l numTopPorts\fR option may be used to select the top
\fInumTopPorts\fR (by bytes of traffic) to be placed in selected port
table objects in the output file.  In other words, instead of writing
port matrix objects, we write selected port table objects where the
selection criteria is the bytes of traffic for each port.  All data for
ports that are not one of the top \fInumTopPorts\fR by bytes of traffic
are lumped into the counters for port 0 in the output selected port
objects.
.IP "\fB-h hours"
The \fI-h hours\fR option may be used to specify the period of
aggregation, in hours.  The \fIhours\fR argument accepts a decimal value
to permit aggregation by fractional hour values.  When the \fI-h
hours\fR option is not used, \fIartsagg\fR will write a single ARTS
object for each ARTS data type for each interface.  These objects will
be aggregates for the entire time interval seen in
\fIinfile(s)\fR.
.IP "\fB-s portlist"
The \fI-s portlist\fR option is used to select a set of ports to be
placed in ArtsSelectedPortTable objects in the output file.  In other
words, instead of writing port matrix objects, we write selected port table
objects.  A selected port table object does not retain the source to
destination port information, but instead lumps traffic into the lower
of the source or destination port for ports in \fIportlist\fR and lumps
those that aren't in \fIportlist\fR into the input counters for port 0.

\fIportlist\fR may contain hyphens (indicating an inclusive range) or
commas.  For example, \fI-s 1-1024,6000,7070\fR would select ports
1 through 1024 (inclusive), port 6000 and port 7070.  All traffic for
ports outside of these values would be lumped into counters for port 0.
.IP "\fB-o"
By default, \fIartsagg\fR appends to the output file, so you may
accumlate aggregate objects in a single file over multiple invocations
of \fIartsagg\fR.  You may override this behavior with the \fI-o\fR
option.  This will cause \fIartsagg\fR to overwrite the output file
instead of appending to it.
.IP "\fB-q"
By default, \fIartsagg\fR prints a '.' for each object it
reads and prints a '+' for each object it writes.  You may suppress this
behavior using the \fI-q\fR (quiet) option.
.SH EXAMPLES
.IP "artsagg -h 1.0 /tmp/artsagg1hr.19980821 arts.19980821"
.SH NOTES
.SH SEE ALSO
artsasagg(l), artsases(l), artsnetagg(l), artsnets(l), artsportmagg(l), 
artsportms(l), artsports(l), artsprotoagg(l), artsprotos(l),
artsnexthopagg(l), artsnexthops(l), artsintfmagg(l), artsintfms(l), 
artstos(l), artsdump(l), artstoc(l)
.SH ACKNOWLEDGEMENTS
\fBartsagg \fRand documentation written by Daniel W. McRobb
.br
CAIDA, August 1998

