//===========================================================================
//  $Name: arts++-1-1-a13 $
//  $Id: AsTrafficTotal.hh,v 1.1.1.1 2002/11/16 00:46:38 rkoga Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  arts++ is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License version 2.1 as published by the Free Software Foundation.
//  arts++ is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ASTRAFFICTOTAL_HH_
#define _ASTRAFFICTOTAL_HH_

extern "C" {
#include "caida_t.h"
}

//----------------------------------------------------------------------------
//                           class AsTrafficTotal                          
//----------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
class AsTrafficTotal
{
public:
  //--------------------------------------------------------------------------
  //                         inline AsTrafficTotal()                         
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  inline AsTrafficTotal()
  {
    this->_as = 0;
    this->_inPkts = 0;
    this->_inBytes = 0;
    this->_outPkts = 0;
    this->_outBytes = 0;
  }

  //--------------------------------------------------------------------------
  //                  inline const uint64_t & InPkts() const                 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  inline const uint64_t & InPkts() const
  {
    return(this->_inPkts);
  }

  //--------------------------------------------------------------------------
  //         inline const uint64_t & InPkts(const uint64_t & inPkts)         
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  inline const uint64_t & InPkts(const uint64_t & inPkts)
  {
    this->_inPkts = inPkts;
    return(this->_inPkts);
  }
  
  //--------------------------------------------------------------------------
  //                 inline const uint64_t & InBytes() const                 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  inline const uint64_t & InBytes() const
  {
    return(this->_inBytes);
  }

  //--------------------------------------------------------------------------
  //        inline const uint64_t & InBytes(const uint64_t & inBytes)        
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  inline const uint64_t & InBytes(const uint64_t & inBytes)
  {
    this->_inBytes = inBytes;
    return(this->_inBytes);
  }
  
  //--------------------------------------------------------------------------
  //                 inline const uint64_t & OutPkts() const                 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  inline const uint64_t & OutPkts() const
  {
    return(this->_outPkts);
  }

  //--------------------------------------------------------------------------
  //        inline const uint64_t & OutPkts(const uint64_t & outPkts)        
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  inline const uint64_t & OutPkts(const uint64_t & outPkts)
  {
    this->_outPkts = outPkts;
    return(this->_outPkts);
  }
  
  //--------------------------------------------------------------------------
  //                 inline const uint64_t & OutBytes() const                
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  inline const uint64_t & OutBytes() const
  {
    return(this->_outBytes);
  }

  //--------------------------------------------------------------------------
  //       inline const uint64_t & OutBytes(const uint64_t & outBytes)       
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  inline const uint64_t & OutBytes(const uint64_t & outBytes)
  {
    this->_outBytes = outBytes;
    return(this->_outBytes);
  }

  //--------------------------------------------------------------------------
  //                    inline const uint16_t & As() const                   
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  inline const uint16_t & As() const
  {
    return(this->_as);
  }

  //--------------------------------------------------------------------------
  //             inline const uint16_t & As(const uint16_t & as)             
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  inline const uint16_t & As(const uint16_t & as)
  {
    this->_as = as;
    return(this->_as);
  }
  
  //--------------------------------------------------------------------------
  //           inline AsTrafficTotal &
  //           operator = (const AsTrafficTotal & asTrafficTotal) 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  inline AsTrafficTotal &
  operator = (const AsTrafficTotal & asTrafficTotal)
  {
    this->_as = asTrafficTotal.As();
    this->_inPkts = asTrafficTotal.InPkts();
    this->_inBytes = asTrafficTotal.InBytes();
    this->_outPkts = asTrafficTotal.OutPkts();
    this->_outBytes = asTrafficTotal.OutBytes();
    return(*this);
  }
  
private:
  uint16_t    _as;
  uint64_t    _inPkts;
  uint64_t    _inBytes;
  uint64_t    _outPkts;
  uint64_t    _outBytes;
};

//---------------------------------------------------------------------------
//  class AsTrafficTotalGreaterBytes
//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class AsTrafficTotalGreaterBytes
{
public:
  bool operator () (const AsTrafficTotal & asTrafficTotal1,
                    const AsTrafficTotal & asTrafficTotal2) const
  {
    return(asTrafficTotal1.InBytes() + asTrafficTotal1.OutBytes() >
           asTrafficTotal2.InBytes() + asTrafficTotal2.OutBytes());
  }
};


//---------------------------------------------------------------------------
//  class AsTrafficTotalGreaterPkts
//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class AsTrafficTotalGreaterPkts
{
public:
  bool operator () (const AsTrafficTotal & asTrafficTotal1,
                    const AsTrafficTotal & asTrafficTotal2) const
  {
    return(asTrafficTotal1.InPkts() + asTrafficTotal1.OutPkts() >
           asTrafficTotal2.InPkts() + asTrafficTotal2.OutPkts());
  }
};


#endif  // _ASTRAFFICTOTAL_HH_
