//===========================================================================
//  @(#) $Name: arts++-1-1-a13 $
//  @(#) $Id: artsintfms.cc,v 1.3 2008/04/14 21:17:07 rkoga Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>
#include <errno.h>
}

#ifdef HAVE_FSTREAM
#include <fstream.h>
#else
#include <fstream>
#endif

#include <string>
#include <cstring> // strdup(), strerror()
#include <iterator>

#include "ArtsInterfaceMatrix.hh"
#include "ArtsIfIndexSelectionSet.hh"
#include "ArtsTimeIntervalSelectionSet.hh"
#include "ArtsPackageVersion.hh"

#ifndef NDEBUG
  #include "ArtsDebug.hh"
#endif

extern char *tzname[2];

using namespace std;

const string rcsid = "@(#) $Name: arts++-1-1-a13 $ $Id: artsintfms.cc,v 1.3 2008/04/14 21:17:07 rkoga Exp $";
static const ArtsPackageVersion  k_artsVersion = ArtsPackageVersion(rcsid);

static bool      g_srcInterfaceSpecified = false;
static bool      g_dstInterfaceSpecified = false;
static uint16_t  g_srcInterface = 0;
static uint16_t  g_dstInterface = 0;
static ArtsTimeIntervalSelectionSet  g_timeIntervalSet;

//-------------------------------------------------------------------------
//   static bool                               
//   InterfaceEntryMatches(vector<ArtsInterfaceMatrixEntry>::const_iterator & interfaceEntry) 
//.........................................................................
//  
//-------------------------------------------------------------------------
static bool
InterfaceEntryMatches(vector<ArtsInterfaceMatrixEntry>::const_iterator & interfaceEntry,
                      ArtsIfIndexSelectionSet & srcIfIndexSet,
                      ArtsIfIndexSelectionSet & dstIfIndexSet)
{
  if (srcIfIndexSet.size() > 0)
    if (! srcIfIndexSet.Matches(interfaceEntry->Src()))
      return(false);
  if (dstIfIndexSet.size() > 0)
    if (! dstIfIndexSet.Matches(interfaceEntry->Dst()))
      return(false);

  return(true);
}

//----------------------------------------------------------------------------
//  static void ShowInterfaceMatrix(const Arts & arts, bool sortByPkts, 
//                                  ArtsIfIndexSelectionSet & ifIndexSet, 
//                                  ArtsIfIndexSelectionSet & srcIfIndexSet, 
//                                  ArtsIfIndexSelectionSet & dstIfIndexSet) 
//............................................................................
//  
//----------------------------------------------------------------------------
static void ShowInterfaceMatrix(const Arts & arts, bool sortByPkts,
                                ArtsIfIndexSelectionSet & ifIndexSet,
                                ArtsIfIndexSelectionSet & srcIfIndexSet,
                                ArtsIfIndexSelectionSet & dstIfIndexSet)
{
  struct tm       *localTm;
  time_t           periodTimes[2];
  struct in_addr   inAddr;
  
  vector<ArtsAttribute>::const_iterator  periodAttribute;
  vector<ArtsAttribute>::const_iterator  hostAttribute;
  vector<ArtsAttribute>::const_iterator  ifIndexAttribute;
  vector<ArtsAttribute>::const_iterator  ifDescrAttribute;
  vector<ArtsAttribute>::const_iterator  ifIpAddrAttribute;
  
  hostAttribute = arts.FindHostAttribute();
  ifIndexAttribute = arts.FindIfIndexAttribute();
  if (ifIndexSet.size() > 0) {
    if (ifIndexAttribute != arts.Attributes().end()) {
      if (! ifIndexSet.Matches(ifIndexAttribute->IfIndex())) {
        return;
      }
    }
    else {
      return;
    }
  }
  periodAttribute = arts.FindPeriodAttribute();
  if (periodAttribute != arts.Attributes().end()) {
    if (g_timeIntervalSet.size() > 0) {
      if ((! g_timeIntervalSet.Matches(periodAttribute->Period()[0])) ||
          (! g_timeIntervalSet.Matches(periodAttribute->Period()[1]))) {
        return;
      }
    }
  }
  
  if (hostAttribute != arts.Attributes().end()) {
    inAddr.s_addr = hostAttribute->Host();
    cout << "router:  " << inet_ntoa(inAddr) << endl;
  }

  if (ifIndexAttribute != arts.Attributes().end()) {
    cout << "ifIndex: " << ifIndexAttribute->IfIndex();
  }
  ifDescrAttribute = arts.FindIfDescrAttribute();
  ifIpAddrAttribute = arts.FindIfIpAddrAttribute();
  if (ifDescrAttribute != arts.Attributes().end() &&
      ifIpAddrAttribute != arts.Attributes().end()) {
    cout << " (" << ifDescrAttribute->IfDescr() << " ";
    inAddr.s_addr = ifIpAddrAttribute->IfIpAddr();
    cout << inet_ntoa(inAddr) << ")";
  }
  cout << endl;
    
  if (periodAttribute != arts.Attributes().end()) {
    periodTimes[0] = periodAttribute->Period()[0];
    periodTimes[1] = periodAttribute->Period()[1];
    localTm = localtime(&(periodTimes[0]));
    cout << "period:  " << setfill('0')
         << setw(2) << (int)localTm->tm_mon+1 << "/"
         << setw(2) << localTm->tm_mday << "/"
         << setw(4) << localTm->tm_year + 1900 << " "
         << setw(2) << localTm->tm_hour << ":"
         << setw(2) << localTm->tm_min << ":"
         << setw(2) << localTm->tm_sec << " - ";
    localTm = localtime(&(periodTimes[1]));
    cout << setw(2) << (int)localTm->tm_mon+1 << "/"
         << setw(2) << localTm->tm_mday << "/"
         << setw(4) << localTm->tm_year + 1900 << " "
         << setw(2) << localTm->tm_hour << ":"
         << setw(2) << localTm->tm_min << ":"
         << setw(2) << localTm->tm_sec << " " << tzname[localTm->tm_isdst]
         << endl;
  }

  time_t timeInterval = periodTimes[1] - periodTimes[0];
  
  cout << setfill(' ');
  cout << setw(9) << "SrcIntf"
       << setw(9) << "DstIntf"
       << setw(15) << "Pkts"
       << setw(15) << "Pkts/sec"
       << setw(15) << "Bytes"
       << setw(15) << "Bits/sec"
       << endl;
  cout << setw(9) << "-------"
       << setw(9) << "-------"
       << setw(15) << "-------------"
       << setw(15) << "-------------"
       << setw(15) << "-------------"
       << setw(15) << "-------------"
       << endl;

  if (sortByPkts)
    arts.InterfaceMatrixData()->SortEntriesByPkts();
  else
    arts.InterfaceMatrixData()->SortEntriesByBytes();

  vector<ArtsInterfaceMatrixEntry>::const_iterator  interfaceEntry;
  for (interfaceEntry = arts.InterfaceMatrixData()->InterfaceEntries().begin();
       interfaceEntry != arts.InterfaceMatrixData()->InterfaceEntries().end();
       interfaceEntry++) {
    if (InterfaceEntryMatches(interfaceEntry,srcIfIndexSet,dstIfIndexSet)) {
      cout << setw(9) << interfaceEntry->Src()
           << setw(9) << interfaceEntry->Dst()
           << setw(15) << interfaceEntry->Pkts()
           << setw(15) << (double)interfaceEntry->Pkts() / timeInterval
           << setw(15) << interfaceEntry->Bytes()
           << setw(15) << ((double)interfaceEntry->Bytes() * 8.0) / timeInterval
           << endl;
    }
  }
  cout << endl;
  
  return;
}
  
//-------------------------------------------------------------------------
//                      void Usage(const char *argv0)                      
//.........................................................................
//  
//-------------------------------------------------------------------------
void Usage(const char *argv0)
{
  cerr << "Usage: " << argv0
       << " [-v] [-p] [-i ifIndexList] [-s srcIntfList] [-d dstIntfList]"
       << " [-b beginTime] [-e endTime] file(s)" << endl;
  return;
}

//-------------------------------------------------------------------------
//                     int main(int argc, char *argv[])                    
//.........................................................................
//  
//-------------------------------------------------------------------------
int main(int argc, char *argv[])
{
  ifstream    *artsStream;
  extern char *optarg;
  extern int   optind;
  int          optChar;
  bool         sortByPkts = false;
  ArtsIfIndexSelectionSet   ifIndexSet;
  ArtsIfIndexSelectionSet   srcIfIndexSet;
  ArtsIfIndexSelectionSet   dstIfIndexSet;
  char                     *startTimeExpression = (char *)0;
  char                     *endTimeExpression = (char *)0;

  
  if (argc < 2) {
    Usage(argv[0]);
    exit(1);
  }

  while ((optChar = getopt(argc,argv,"b:e:i:ps:d:v")) != EOF) {
    switch (optChar) {
      case 'b':
        startTimeExpression = (char *)strdup(optarg);
        break;
      case 'e':
        endTimeExpression = (char *)strdup(optarg);
        break;
      case 'i':
        ifIndexSet.Load(optarg);
        break;
      case 'p':
        sortByPkts = true;
        break;
      case 's':
        srcIfIndexSet.Load(optarg);
        break;
      case 'd':
        dstIfIndexSet.Load(optarg);
        break;
      case 'v':
        cerr << k_artsVersion.Name() << "  " << k_artsVersion.Id() << endl;
        exit(0);
        break;
      case '?':
      default:
        Usage(argv[0]);
        exit(1);
        break;
    }
  }

  if (optind >= argc) {
    Usage(argv[0]);
    exit(1);
  }
  
  if (startTimeExpression || endTimeExpression)
    g_timeIntervalSet.Load(startTimeExpression,endTimeExpression);

  for (int fileNum = optind; fileNum < argc; fileNum++) {
    artsStream = new ifstream(argv[fileNum]);
    
    if (! artsStream || !*artsStream) {
      cerr << "[E] unable to open input file '" << argv[fileNum] << "': "
           << strerror(errno) << endl;
      continue;
    }

    istream_iterator<ArtsInterfaceMatrix>  inStreamIter(*artsStream);
    istream_iterator<ArtsInterfaceMatrix>  inStreamEnd;

    for (; inStreamIter != inStreamEnd; inStreamIter++) {
      if ((*inStreamIter).InterfaceMatrixData()->InterfaceEntries().size()
          > 0) {
        ShowInterfaceMatrix(*inStreamIter,sortByPkts,ifIndexSet,
                            srcIfIndexSet,dstIfIndexSet);
      }
    } //.......... for (; inStreamIter != inStreamEnd; inStreamIter++
    
    delete(artsStream);
    
  } //.......... for (int fileNum = optind; ...)

  #ifndef NDEBUG
    atexit(ShowArtsObjectsNotFreed);
  #endif
  
  exit(0);
}
