//===========================================================================
//  @(#) $Name: arts++-1-1-a13 $
//  @(#) $Id: artstrunc.cc,v 1.2 2004/04/21 23:51:24 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>
#include <errno.h>
}

#ifdef HAVE_FSTREAM
#include <fstream.h>
#else
#include <fstream>
#endif

#include <string>

#include "Arts.hh"
#include "ArtsTimeIntervalSelectionSet.hh"
#include "ArtsPackageVersion.hh"
#ifndef NDEBUG
  #include "ArtsDebug.hh"
#endif

extern char *tzname[2];

using namespace std;

static const string rcsid = "@(#) $Name: arts++-1-1-a13 $ $Id: artstrunc.cc,v 1.2 2004/04/21 23:51:24 kkeys Exp $";
static const ArtsPackageVersion  k_artsVersion = ArtsPackageVersion(rcsid);
  
//-------------------------------------------------------------------------
//                      void Usage(const char *argv0)                      
//.........................................................................
//  
//-------------------------------------------------------------------------
void Usage(const char *argv0)
{
  cerr << "Usage: " << argv0 << " [-v] [-b] file"
       << endl;
  return;
}

//-------------------------------------------------------------------------
//                     int main(int argc, char *argv[])                    
//.........................................................................
//  
//-------------------------------------------------------------------------
int main(int argc, char *argv[])
{
  Arts         arts;
  extern char *optarg;
  extern int   optind;
  int          optChar;
  bool         bruteForce = false;
  
  if (argc < 2) {
    Usage(argv[0]);
    exit(1);
  }

  while ((optChar = getopt(argc,argv,"bv")) != EOF) {
    switch (optChar) {
      case 'b':
        bruteForce = true;
        break;
      case 'v':
        cerr << k_artsVersion.Name() << "  " << k_artsVersion.Id() << endl;
        exit(0);
        break;
      case '?':
      default:
        Usage(argv[0]);
        exit(1);
        break;
    }
  }

  if (optind >= argc) {
    Usage(argv[0]);
    exit(1);
  }

  int fileNum = optind;

  ifstream artsStream(argv[fileNum]);
  if (! artsStream) {
    cerr << "[E] unable to open input file '" << argv[fileNum] << "': "
         << strerror(errno) << endl;
    Usage(argv[0]);
    exit(1);
  }

  struct stat statBuf;
  stat(argv[fileNum],&statBuf);
  
  streampos  lastGoodOffset;
  uint8_t    objectsRead = 0;
  
  while (artsStream) {
    lastGoodOffset = artsStream.tellg();
    if (! bruteForce) {
      if (! arts.Header().read(artsStream))
        break;
      if (! artsStream.seekg(arts.Header().AttrLength(),ios::cur))
        break;
      if (! artsStream.seekg(arts.Header().DataLength(),ios::cur))
        break;
    }
    else {
      if (! arts.read(artsStream))
        break;
    }

    if (artsStream.tellg() > statBuf.st_size)
      break;
    
    objectsRead++;
  }
  cout << "lastGoodOffset: " << lastGoodOffset << endl;
  artsStream.close();

  truncate(argv[fileNum],lastGoodOffset);
  
  exit(0);
}
