//===========================================================================
//  @(#) $Name: arts++-1-1-a13 $
//  @(#) $Id: ArtsTosTableAggregator.hh,v 1.2 2004/04/21 23:51:30 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSTOSTABLEAGGREGATOR_HH_
#define _ARTSTOSTABLEAGGREGATOR_HH_

#include <map>
#include <vector>

#include "ArtsTosTable.hh"

//---------------------------------------------------------------------------
//  class ArtsTosTableAggregator
//---------------------------------------------------------------------------
//!  This class is used to aggregate TOS table objects in the time
//!  domain.
//---------------------------------------------------------------------------
class ArtsTosTableAggregator
{
public: 
  //--------------------------------------------------------------------------
  //!  Holds a packet and byte counter.
  //--------------------------------------------------------------------------
  typedef struct {
    uint64_t  Pkts;
    uint64_t  Bytes;
  } counter_t;
  
 //-------------------------------------------------------------------------
  //               ArtsTosTableAggregator(const Arts & arts)
  //.........................................................................
  //!  Construct from an Arts object.  \c arts must be a TOS table
  //!  object.
  //-------------------------------------------------------------------------
  ArtsTosTableAggregator(const Arts & arts);

  ~ArtsTosTableAggregator();
  
  //-------------------------------------------------------------------------
  //                       void Add(const Arts & arts)
  //.........................................................................
  //!  Adds the contents of an Arts object.  The input Arts object must be
  //!  a TOS table object.  The effect here is to add any new TOS table
  //!  entries and add to the counters for existing TOS table entries.
  //-------------------------------------------------------------------------
  void Add(const Arts & arts);
  
  //-------------------------------------------------------------------------
  //              ArtsTosTable *ConvertToArtsTosTable() const
  //.........................................................................
  //!  Return an ArtsTosTable object (dynamically allocated) containing
  //!  the aggregate data.  The caller is responsible for calling delete()
  //!  on the returned ArtsTosTable pointer.
  //-------------------------------------------------------------------------
  ArtsTosTable *ConvertToArtsTosTable() const;
  
private:
  ArtsHeader                               _header;
  std::vector<ArtsAttribute>                    _attributes;
 std::map<uint8_t,counter_t,std::less<uint8_t> >    _tosCounters;
  
};

#endif  // _ARTSTOSTABLEAGGREGATOR_HH_
