//===========================================================================
// @(#) $Name: arts++-1-1-a13 $
// @(#) $Id: ArtsAttributeVector.cc,v 1.2 2004/04/21 23:51:31 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#include <string>

#include "ArtsAttributeVector.hh"

using namespace std;

static const std::string rcsid = "@(#) $Name: arts++-1-1-a13 $ $Id: ArtsAttributeVector.cc,v 1.2 2004/04/21 23:51:31 kkeys Exp $";

//----------------------------------------------------------------------------
//     void ArtsAttributeVector::AddHostAttribute(ipv4addr_t hostAddr) 
//............................................................................
//  
//----------------------------------------------------------------------------
void ArtsAttributeVector::AddHostAttribute(ipv4addr_t hostAddr)
{
  ArtsAttribute  attribute;

  attribute.Identifier(artsC_ATTR_HOST);
  attribute.Host(hostAddr);
  attribute.Format(artsC_IPV4_ADDRESS);
  attribute.Length(sizeof(uint32_t) + sizeof(uint32_t) + sizeof(ipv4addr_t));

  this->push_back(attribute);

  return;
}

//----------------------------------------------------------------------------
//      const_iterator ArtsAttributeVector::FindHostAttribute() const 
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsAttributeVector::const_iterator
ArtsAttributeVector::FindHostAttribute() const
{
  return(this->Find(artsC_ATTR_HOST));
}

//----------------------------------------------------------------------------
//             void ArtsAttributeVector::RemoveHostAttribute() 
//............................................................................
//  
//----------------------------------------------------------------------------
bool ArtsAttributeVector::RemoveHostAttribute()
{
  return(this->Remove(artsC_ATTR_HOST));
}

//----------------------------------------------------------------------------
//     void ArtsAttributeVector::AddHostPairAttribute(ipv4addr_t hostAddr1, 
//                                                    ipv4addr_t hostAddr2) 
//............................................................................
//  
//----------------------------------------------------------------------------
void ArtsAttributeVector::AddHostPairAttribute(ipv4addr_t hostAddr1,
                                               ipv4addr_t hostAddr2)
{
  ArtsAttribute  attribute;
  attribute.Identifier(artsC_ATTR_HOSTPAIR);
  attribute.HostPair(hostAddr1,hostAddr2);
  attribute.Format(artsC_IPV4_ADDRESS_PAIR);
  attribute.Length(sizeof(uint32_t) + sizeof(uint32_t) +
                   sizeof(ipv4addr_t) * 2);
  this->push_back(attribute);
  return;
}

//----------------------------------------------------------------------------
//         ArtsAttributeVector::const_iterator
//         ArtsAttributeVector::FindHostPairAttribute() const 
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsAttributeVector::const_iterator
ArtsAttributeVector::FindHostPairAttribute() const
{
  return(this->Find(artsC_ATTR_HOSTPAIR));
}

//----------------------------------------------------------------------------
//     bool ArtsAttributeVector::RemoveHostPairAttribute() 
//............................................................................
//  
//----------------------------------------------------------------------------
bool ArtsAttributeVector::RemoveHostPairAttribute()
{
  return(this->Remove(artsC_ATTR_HOSTPAIR));
}


//----------------------------------------------------------------------------
//   void ArtsAttributeVector::AddCreationAttribute(uint32_t creationTime) 
//............................................................................
//  
//----------------------------------------------------------------------------
void ArtsAttributeVector::AddCreationAttribute(uint32_t creationTime)
{
  ArtsAttribute  attribute;

  attribute.Identifier(artsC_ATTR_CREATION);
  attribute.Creation(creationTime);
  attribute.Format(artsC_UNIX_DATE);
  attribute.Length(sizeof(uint32_t) + sizeof(uint32_t) + sizeof(uint32_t));

  this->push_back(attribute);

  return;
}

//----------------------------------------------------------------------------
//    const_iterator ArtsAttributeVector::FindCreationAttribute() const 
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsAttributeVector::const_iterator
ArtsAttributeVector::FindCreationAttribute() const
{
  return(this->Find(artsC_ATTR_CREATION));
}

//----------------------------------------------------------------------------
//           bool ArtsAttributeVector::RemoveCreationAttribute() 
//............................................................................
//  
//----------------------------------------------------------------------------
bool ArtsAttributeVector::RemoveCreationAttribute()
{
  return(this->Remove(artsC_ATTR_CREATION));
}

//----------------------------------------------------------------------------
//     void ArtsAttributeVector::AddPeriodAttribute(uint32_t startTime, 
//                                                  uint32_t endTime) 
//............................................................................
//  
//----------------------------------------------------------------------------
void ArtsAttributeVector::AddPeriodAttribute(uint32_t startTime,
                                             uint32_t endTime)
{
  ArtsAttribute  attribute;

  attribute.Identifier(artsC_ATTR_PERIOD);
  attribute.Period(startTime,endTime);
  attribute.Length(sizeof(uint32_t) + sizeof(uint32_t) +
                   sizeof(uint32_t) + sizeof(uint32_t));
  this->push_back(attribute);

  return;
}

//----------------------------------------------------------------------------
//     const_iterator ArtsAttributeVector::FindPeriodAttribute() const 
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsAttributeVector::const_iterator
ArtsAttributeVector::FindPeriodAttribute() const
{
  return(this->Find(artsC_ATTR_PERIOD));
}
  
//----------------------------------------------------------------------------
//            bool ArtsAttributeVector::RemovePeriodAttribute() 
//............................................................................
//  
//----------------------------------------------------------------------------
bool ArtsAttributeVector::RemovePeriodAttribute()
{
  return(this->Remove(artsC_ATTR_PERIOD));
}

//----------------------------------------------------------------------------
//     void ArtsAttributeVector::AddIfIndexAttribute(uint16_t ifIndex) 
//............................................................................
//  
//----------------------------------------------------------------------------
void ArtsAttributeVector::AddIfIndexAttribute(uint16_t ifIndex)
{
  ArtsAttribute   attribute;
  
  attribute.Identifier(artsC_ATTR_IFINDEX);
  attribute.IfIndex(ifIndex);
  attribute.Length(sizeof(uint32_t) + sizeof(uint32_t) + sizeof(ifIndex));
  this->push_back(attribute);

  return;
}

//----------------------------------------------------------------------------
//     const_iterator ArtsAttributeVector::FindIfIndexAttribute() const 
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsAttributeVector::const_iterator
ArtsAttributeVector::FindIfIndexAttribute() const
{
  return(this->Find(artsC_ATTR_IFINDEX));
}

//----------------------------------------------------------------------------
//           bool ArtsAttributeVector::RemoveIfIndexAttribute() 
//............................................................................
//  
//----------------------------------------------------------------------------
bool ArtsAttributeVector::RemoveIfIndexAttribute()
{
  return(this->Remove(artsC_ATTR_IFINDEX));
}

//----------------------------------------------------------------------------
//  void ArtsAttributeVector::AddIfDescrAttribute(const std::string & ifDescr) 
//............................................................................
//  
//----------------------------------------------------------------------------
void ArtsAttributeVector::AddIfDescrAttribute(const std::string & ifDescr)
{
  ArtsAttribute  attribute;

  attribute.Identifier(artsC_ATTR_IFDESCR);
  attribute.IfDescr(ifDescr);
  attribute.Length(sizeof(uint32_t) + sizeof(uint32_t) + ifDescr.length() + 1);
  this->push_back(attribute);

  return;
}

//----------------------------------------------------------------------------
//     const_iterator ArtsAttributeVector::FindIfDescrAttribute() const 
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsAttributeVector::const_iterator
ArtsAttributeVector::FindIfDescrAttribute() const
{
  return(this->Find(artsC_ATTR_IFDESCR));
}

//----------------------------------------------------------------------------
//            bool ArtsAttributeVector::RemoveIfDescrAttribute() 
//............................................................................
//  
//----------------------------------------------------------------------------
bool ArtsAttributeVector::RemoveIfDescrAttribute()
{
  return(this->Remove(artsC_ATTR_IFDESCR));
}

//----------------------------------------------------------------------------
//    void ArtsAttributeVector::AddIfIpAddrAttribute(ipv4addr_t ipAddr) 
//............................................................................
//  
//----------------------------------------------------------------------------
void ArtsAttributeVector::AddIfIpAddrAttribute(ipv4addr_t ipAddr)
{
  ArtsAttribute  attribute;

  attribute.Identifier(artsC_ATTR_IFIPADDR);
  attribute.IfIpAddr(ipAddr);
  attribute.Length(sizeof(uint32_t) + sizeof(uint32_t) + sizeof(ipAddr));
  this->push_back(attribute);

  return;
}

//----------------------------------------------------------------------------
//    const_iterator ArtsAttributeVector::FindIfIpAddrAttribute() const 
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsAttributeVector::const_iterator
ArtsAttributeVector::FindIfIpAddrAttribute() const
{
  return(this->Find(artsC_ATTR_IFIPADDR));
}

//----------------------------------------------------------------------------
//           bool ArtsAttributeVector::RemoveIfIpAddrAttribute() 
//............................................................................
//  
//----------------------------------------------------------------------------
bool ArtsAttributeVector::RemoveIfIpAddrAttribute()
{
  return(this->Remove(artsC_ATTR_IFIPADDR));
}

//----------------------------------------------------------------------------
//                  uint32_t ArtsAttributeVector::Length() 
//............................................................................
//  
//----------------------------------------------------------------------------
uint32_t ArtsAttributeVector::Length()
{
  uint32_t        rval = 0;
  const_iterator  attrIter;
  
  for (attrIter = this->begin(); attrIter != this->end(); attrIter++) {
    rval += attrIter->Length();
  }

  return(rval);
}

//----------------------------------------------------------------------------
// std::istream & ArtsAttributeVector::read(std::istream & is, uint16_t numAttributes) 
//............................................................................
//  
//----------------------------------------------------------------------------
std::istream & ArtsAttributeVector::read(std::istream & is, uint16_t numAttributes)
{
  ArtsAttribute  attribute;
  uint16_t       attrNum;

  if (! this->empty())
    this->clear();
  
  if (numAttributes > 0) {
    this->reserve(numAttributes);
    for (attrNum = 0; attrNum < numAttributes; attrNum++) {
      attribute.read(is);
      if (is.eof())
        break;
      this->push_back(attribute);
    }
  }
  return(is);
}

//----------------------------------------------------------------------------
//      int ArtsAttributeVector::read(int fd, uint16_t numAttributes) 
//............................................................................
//  
//----------------------------------------------------------------------------
int ArtsAttributeVector::read(int fd, uint16_t numAttributes)
{
  ArtsAttribute  attribute;
  uint16_t       attrNum;
  int             rc;
  int             bytesRead = 0;

  if (! this->empty())
    this->clear();
  
  if (numAttributes > 0) {
    this->reserve(numAttributes);
    for (attrNum = 0; attrNum < numAttributes; attrNum++) {
      rc = attribute.read(fd);
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      this->push_back(attribute);
    }
  }
  return(bytesRead);
}

//----------------------------------------------------------------------------
//            std::ostream & ArtsAttributeVector::write(std::ostream & os) 
//............................................................................
//  
//----------------------------------------------------------------------------
std::ostream & ArtsAttributeVector::write(std::ostream & os)
{
  if (this->size() > 0) {
    iterator  attrIter;
  
    for (attrIter = this->begin(); attrIter != this->end(); attrIter++) {
      attrIter->write(os);
    }
  }
  return(os);
}


//----------------------------------------------------------------------------
//                  int ArtsAttributeVector::write(int fd) 
//............................................................................
//  
//----------------------------------------------------------------------------
int ArtsAttributeVector::write(int fd)
{
  int  rc;
  int  bytesWritten = 0;

  if (this->size() > 0) {
    iterator  attrIter;
    for (attrIter = this->begin(); attrIter != this->end(); attrIter++) {
      rc = attrIter->write(fd);
      if (rc <= 0)
        return(rc);
      bytesWritten += rc;
    }
  }
  return(bytesWritten);
}

//----------------------------------------------------------------------------
//    const_iterator ArtsAttributeVector::Find(uint32_t attrType) const
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsAttributeVector::const_iterator
ArtsAttributeVector::Find(uint32_t attrType) const
{
  const_iterator attrIter;
  for (attrIter = this->begin(); attrIter != this->end(); attrIter++) {
    if ((*attrIter).Identifier() == attrType) {
      return(attrIter);
    }
  }

  return(this->end());
}

//----------------------------------------------------------------------------
//           bool ArtsAttributeVector::Remove(uint32_t attrType) 
//............................................................................
//  
//----------------------------------------------------------------------------
bool ArtsAttributeVector::Remove(uint32_t attrType)
{
  iterator  attrIter;
  bool      rc = false;
  
  for (attrIter = this->begin(); attrIter != this->end(); attrIter++) {
    if ((*attrIter).Identifier() == attrType) {
      this->erase(attrIter);
      rc = true;
      break;
    }
  }
  return(rc);
}
