//===========================================================================
//  $Name: arts++-1-1-a13 $
//  $Id: ArtsBgp4AggregatorAttribute.cc,v 1.2 2004/04/21 23:51:31 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#include <string>

#include "ArtsPrimitive.hh"
#include "ArtsBgp4AggregatorAttribute.hh"

using namespace std;

static const std::string rcsid = "@(#) $Name: arts++-1-1-a13 $ $Id: ArtsBgp4AggregatorAttribute.cc,v 1.2 2004/04/21 23:51:31 kkeys Exp $";

//----------------------------------------------------------------------------
//        ArtsBgp4AggregatorAttribute::ArtsBgp4AggregatorAttribute()       
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsBgp4AggregatorAttribute::ArtsBgp4AggregatorAttribute()
{
  this->_AS = 0;
  this->_ipAddr = 0;

  #ifndef NDEBUG
    ++this->_numObjects;
  #endif
}

//----------------------------------------------------------------------------
//  ArtsBgp4AggregatorAttribute:: 
//  ArtsBgp4AggregatorAttribute(const ArtsBgp4AggregatorAttribute & aggAttr) 
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsBgp4AggregatorAttribute::
ArtsBgp4AggregatorAttribute(const ArtsBgp4AggregatorAttribute & aggAttr)
{
  this->AS(aggAttr.AS());
  this->IpAddr(aggAttr.IpAddr());
}

//----------------------------------------------------------------------------
//       ArtsBgp4AggregatorAttribute::~ArtsBgp4AggregatorAttribute()       
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsBgp4AggregatorAttribute::~ArtsBgp4AggregatorAttribute()
{
  #ifndef NDEBUG
    --this->_numObjects;
  #endif
}

//-------------------------------------------------------------------------
//           uint16_t ArtsBgp4AggregatorAttribute::AS() const            
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint16_t ArtsBgp4AggregatorAttribute::AS() const
{
  return(this->_AS);
}

//-------------------------------------------------------------------------
//         uint16_t ArtsBgp4AggregatorAttribute::AS(uint16_t as)         
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint16_t ArtsBgp4AggregatorAttribute::AS(uint16_t as)
{
  this->_AS = as;
  return(this->_AS);
}

//----------------------------------------------------------------------------
//         ipv4addr_t & ArtsBgp4AggregatorAttribute::IpAddr() const        
//............................................................................
//  
//----------------------------------------------------------------------------
const ipv4addr_t & ArtsBgp4AggregatorAttribute::IpAddr() const
{
  return(this->_ipAddr);
}

//----------------------------------------------------------------------------
//        const ipv4addr_t &
//        ArtsBgp4AggregatorAttribute::IpAddr(ipv4addr_t & ipAddr)  
//............................................................................
//  
//----------------------------------------------------------------------------
const ipv4addr_t &
ArtsBgp4AggregatorAttribute::IpAddr(const ipv4addr_t & ipAddr)
{
  this->_ipAddr = ipAddr;
  return(this->_ipAddr);
}

//-------------------------------------------------------------------------
// ArtsBgp4AggregatorAttribute &                      
// ArtsBgp4AggregatorAttribute::operator = (const ArtsBgp4AggregatorAttribute &
//                                          aggregator) 
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ArtsBgp4AggregatorAttribute &
ArtsBgp4AggregatorAttribute::operator = (const ArtsBgp4AggregatorAttribute &
                                         aggregator)
{
  this->AS(aggregator.AS());
  this->IpAddr(aggregator.IpAddr());
  return(*this);
}

//----------------------------------------------------------------------------
//        std::istream & ArtsBgp4AggregatorAttribute::read(std::istream & is,        
//                                                    uint8_t version) 
//............................................................................
//  
//----------------------------------------------------------------------------
std::istream & ArtsBgp4AggregatorAttribute::read(std::istream & is,
                                            uint8_t version)
{
  g_ArtsLibInternal_Primitive.ReadUint16(is,this->_AS,sizeof(this->_AS));
  is.read((char*)&this->_ipAddr,sizeof(this->_ipAddr));
  return(is);
}

//----------------------------------------------------------------------------
//    int ArtsBgp4AggregatorAttribute::read(int fd, uint8_t version)   
//............................................................................
//  
//----------------------------------------------------------------------------
int ArtsBgp4AggregatorAttribute::read(int fd, uint8_t version)
{
  int  rc;
  int  bytesRead = 0;
  
  rc = g_ArtsLibInternal_Primitive.ReadUint16(fd,this->_AS,sizeof(this->_AS));
  if (rc < sizeof(this->_AS))
    return(-1);
  bytesRead += rc;
  rc = g_ArtsLibInternal_Primitive.ReadIpv4Network(fd,this->_ipAddr,
                                                   sizeof(this->_ipAddr));
  if (rc < sizeof(this->_ipAddr))
    return(-1);
  bytesRead += rc;
  return(bytesRead);
}

//---------------------------------------------------------------------------
//  std::ostream & ArtsBgp4AggregatorAttribute::write(std::ostream & os,
//                                               uint8_t version) const
//............................................................................
//
//----------------------------------------------------------------------------
std::ostream & ArtsBgp4AggregatorAttribute::write(std::ostream & os,
                                             uint8_t version) const
{
  g_ArtsLibInternal_Primitive.WriteUint16(os,this->_AS,sizeof(this->_AS));
  os.write((char*)&this->_ipAddr,sizeof(this->_ipAddr));
  return(os);
}

//----------------------------------------------------------------------------
// int ArtsBgp4AggregatorAttribute::write(int fd, uint8_t version) const 
//............................................................................
//  
//----------------------------------------------------------------------------
int ArtsBgp4AggregatorAttribute::write(int fd, uint8_t version) const
{
  int  rc;
  int  bytesWritten = 0;

  rc = g_ArtsLibInternal_Primitive.WriteUint16(fd,this->_AS,
                                               sizeof(this->_AS));
  if (rc < sizeof(this->_AS))
    return(-1);
  bytesWritten += rc;
  rc = g_ArtsLibInternal_Primitive.WriteIpv4Network(fd,this->_ipAddr,
                                                    sizeof(this->_ipAddr));
  if (rc < sizeof(this->_ipAddr))
    return(-1);
  bytesWritten += rc;
  return(bytesWritten);
}

//----------------------------------------------------------------------------
// uint32_t ArtsBgp4AggregatorAttribute::Length(uint8_t version) const 
//............................................................................
//  
//----------------------------------------------------------------------------
uint32_t ArtsBgp4AggregatorAttribute::Length(uint8_t version) const
{
  return(sizeof(this->_AS) + sizeof(this->_ipAddr));
}

uint32_t ArtsBgp4AggregatorAttribute::_numObjects = 0;
