//===========================================================================
//  @(#) $Name: arts++-1-1-a13 $
//  @(#) $Id: ArtsBgp4RouteTable.cc,v 1.2 2004/04/21 23:51:32 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#include <string>
#include "ArtsBgp4RouteTable.hh"

using namespace std;

static const std::string rcsid = "@(#) $Name: arts++-1-1-a13 $ $Id: ArtsBgp4RouteTable.cc,v 1.2 2004/04/21 23:51:32 kkeys Exp $";

//----------------------------------------------------------------------------
//                 ArtsBgp4RouteTable::ArtsBgp4RouteTable()                
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsBgp4RouteTable::ArtsBgp4RouteTable()
{
  this->Header().Identifier(artsC_OBJECT_BGP4);
  this->_data._bgp4RouteTable = new ArtsBgp4RouteTableData();
}

//----------------------------------------------------------------------------
//                ArtsBgp4RouteTable::~ArtsBgp4RouteTable()                
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsBgp4RouteTable::~ArtsBgp4RouteTable()
{
}


//----------------------------------------------------------------------------
//      istream & operator >> (istream & is, 
//                             ArtsBgp4RouteTable & artsBgp4RouteTable) 
//............................................................................
//  
//----------------------------------------------------------------------------
istream & operator >> (istream & is,
                       ArtsBgp4RouteTable & artsBgp4RouteTable)
{
  streampos   streamPosition;
  ArtsHeader  artsHeader;

  //  save our stream position
  streamPosition = is.tellg();

  //  read the object header
  artsHeader.read(is);
  if (is.eof()) {
    return(is);
  }

  //  skip over objects that are not BGP4 route table objects
  while (artsHeader.Identifier() != artsC_OBJECT_BGP4) {
    is.seekg(artsHeader.AttrLength() + artsHeader.DataLength(),ios::cur);
    streamPosition = is.tellg();
    artsHeader.read(is);
    if (is.eof()) {
      break;
    }
  }

  //  if we reached the end of the file before finding a BGP4 route
  //  table object, just return the istream and don't touch
  //  artsBgp4RouteTable
  if (is.eof()) {
    return(is);
  }

  //  back up to the start of the header and read the entire BGP4 route
  //  table object, then return the istream.
  is.seekg(streamPosition,ios::beg);
  artsBgp4RouteTable.read(is);
  return(is);
}
