%{
//===========================================================================
// @(#) $Name: arts++-1-1-a13 $
// @(#) $Id: ArtsIfIndexSelectionSet.lex,v 1.2 2004/04/21 23:51:32 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <stdio.h>

#include "artslocal.h"
}

#include <string>

#if defined(HAVE_SSTREAM)
  #include <sstream>
#elif defined(HAVE_STRSTREAM)
  #include <strstream>
#else
  #include <strstream.h>
#endif

#include "ArtsIfIndexSelectionSet.hh"

using namespace std;

static const string rcsid = "@(#) $Name: arts++-1-1-a13 $ $Id: ArtsIfIndexSelectionSet.lex,v 1.2 2004/04/21 23:51:32 kkeys Exp $";

%}

%x VALUERANGE
%x SINGLEVALUE
%x ALLVALUES

%option noyywrap

%%

<INITIAL>[0-9]+[-][0-9]+  {return(VALUERANGE);}
<INITIAL>[0-9]+           {return(SINGLEVALUE);}
<INITIAL>all              {return(ALLVALUES);}
.

%%

//----------------------------------------------------------------------------
// void ArtsIfIndexSelectionSet::Load(const char *ifIndexSelectionExpression) 
//............................................................................
//  
//----------------------------------------------------------------------------
void ArtsIfIndexSelectionSet::Load(const char *ifIndexSelectionExpression)
{
  int                  test;
  int                  firstIfIndex, lastIfIndex, portNum;
#ifdef HAVE_SSTREAM
  std::istringstream   inStream(ifIndexSelectionExpression);
#else
  std::istrstream      inStream(ifIndexSelectionExpression);
#endif

  IfIndexFlexLexer *lexer = new IfIndexFlexLexer(&inStream,0);
  
  while ((test = lexer->yylex())) {
    switch (test) {
      case VALUERANGE:
        sscanf(lexer->YYText(),"%d-%d",&firstIfIndex,&lastIfIndex);
        this->AddRange(firstIfIndex,lastIfIndex);
        break;
      case SINGLEVALUE:
        this->Add(atoi(lexer->YYText()));
        break;
      case ALLVALUES:
        this->AddRange(0,0xffff);
        break;
      default:
        break;
    }
  }
  delete(lexer);

  return;
}
