//===========================================================================
//  @(#) $Name: arts++-1-1-a13 $
//  @(#) $Id: ArtsInterfaceMatrix.cc,v 1.2 2004/04/21 23:51:32 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#include <string>

#include "Arts.hh"
#include "ArtsInterfaceMatrix.hh"

using namespace std;

static const std::string rcsid = "@(#) $Name: arts++-1-1-a13 $ $Id: ArtsInterfaceMatrix.cc,v 1.2 2004/04/21 23:51:32 kkeys Exp $";

//-------------------------------------------------------------------------
//               ArtsInterfaceMatrix::ArtsInterfaceMatrix()
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ArtsInterfaceMatrix::ArtsInterfaceMatrix()
{
  this->Header().Identifier(artsC_OBJECT_INTERFACE_MATRIX);
  this->_data._interfaceMatrix = new ArtsInterfaceMatrixData();
}

//-------------------------------------------------------------------------
//              ArtsInterfaceMatrix::~ArtsInterfaceMatrix()
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ArtsInterfaceMatrix::~ArtsInterfaceMatrix()
{
}

//----------------------------------------------------------------------------
//          istream & 
//          operator >> (istream & is, 
//                       ArtsInterfaceMatrix & artsInterfaceMatrix) 
//............................................................................
//  
//----------------------------------------------------------------------------
istream &
operator >> (istream & is,
             ArtsInterfaceMatrix & artsInterfaceMatrix)
{
  streampos   streamPosition;
  ArtsHeader  artsHeader;

  //  save our stream position
  streamPosition = is.tellg();

  //  read the object header
  artsHeader.read(is);
  if (is.eof()) {
    return(is);
  }

  //  skip over objects that are not interface matrix objects
  while (artsHeader.Identifier() != artsC_OBJECT_INTERFACE_MATRIX) {
    is.seekg(artsHeader.AttrLength() + artsHeader.DataLength(),ios::cur);
    streamPosition = is.tellg();
    artsHeader.read(is);
    if (is.eof()) {
      break;
    }
  }

  //  if we reached the end of the file before finding an interface
  //  matrix object, just return the istream and don't touch
  //  artsInterfaceMatrix
  if (is.eof()) {
    return(is);
  }

  //  back up to the start os the header and read the entire interface
  //  matrix object, then return the istream.
  is.seekg(streamPosition,ios::beg);
  artsInterfaceMatrix.read(is);
  return(is);
}
  
