//===========================================================================
//  @(#) $Name: arts++-1-1-a13 $
//  @(#) $Id: ArtsIpPath.cc,v 1.2 2004/04/21 23:51:32 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#include <string>
#include <algorithm>

#include "Arts.hh"
#include "ArtsIpPath.hh"

using namespace std;

static const std::string rcsid = "@(#) $Name: arts++-1-1-a13 $ $Id: ArtsIpPath.cc,v 1.2 2004/04/21 23:51:32 kkeys Exp $";

//-------------------------------------------------------------------------
//          ArtsIpPath::ArtsIpPath(ipv4addr_t src, ipv4addr_t dst)         
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ArtsIpPath::ArtsIpPath(ipv4addr_t src, ipv4addr_t dst)
{
  this->Header().Identifier(artsC_OBJECT_IP_PATH);
  this->_data._ipPath = new ArtsIpPathData();
  this->IpPathData()->Src(src);
  this->IpPathData()->Dst(dst);
}

//-------------------------------------------------------------------------
//          ArtsIpPath::ArtsIpPath(const ArtsIpPath & artsIpPath)          
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ArtsIpPath::ArtsIpPath(const ArtsIpPath & artsIpPath) 
{
  this->_header = artsIpPath.Header();
  this->_attributes = artsIpPath.Attributes();
  this->_data._ipPath = new ArtsIpPathData();
  *(this->_data._ipPath) = *(artsIpPath.IpPathData());
}

//-------------------------------------------------------------------------
//                         ArtsIpPath::ArtsIpPath()                        
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ArtsIpPath::ArtsIpPath()
{
  this->Header().Identifier(artsC_OBJECT_IP_PATH);
  this->_data._ipPath = new ArtsIpPathData();
  this->IpPathData()->Dst(0);
}

//-------------------------------------------------------------------------
//                        ArtsIpPath::~ArtsIpPath()                        
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ArtsIpPath::~ArtsIpPath()
{
}

//----------------------------------------------------------------------------
//      std::istream & operator >> (std::istream & is, ArtsIpPath & artsIpPath) 
//............................................................................
//  
//----------------------------------------------------------------------------
std::istream & operator >> (std::istream & is, ArtsIpPath & artsIpPath)
{
  std::streampos   streamPosition;
  ArtsHeader  artsHeader;

  //  save our stream position
  streamPosition = is.tellg();

  //  read the object header
  artsHeader.read(is);
  if (is.eof()) {
    return(is);
  }

  //  skip over objects that are not IP path objects
  while (artsHeader.Identifier() != artsC_OBJECT_IP_PATH) {
    is.seekg(artsHeader.AttrLength() + artsHeader.DataLength(),std::ios::cur);
    streamPosition = is.tellg();
    artsHeader.read(is);
    if (is.eof()) {
      break;
    }
  }

  //  if we reached the end of the file before finding an IP path object,
  //  just return the std::istream and don't touch artsIpPath
  if (is.eof()) {
    return(is);
  }

  //  back up to the start os the header and read the entire IP path
  //  object, then return the std::istream.
  is.seekg(streamPosition,std::ios::beg);
  artsIpPath.read(is);
  return(is);
}
  

