//===========================================================================
// @(#) $Name: arts++-1-1-a13 $
// @(#) $Id: ArtsNextHopTableAggregator.cc,v 1.2 2004/04/21 23:51:33 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#include <string>

#include "ArtsNextHopTableAggregator.hh"

using namespace std;

static const std::string rcsid = "@(#) $Name: arts++-1-1-a13 $ $Id: ArtsNextHopTableAggregator.cc,v 1.2 2004/04/21 23:51:33 kkeys Exp $";

//-------------------------------------------------------------------------
// ArtsNextHopTableAggregator::ArtsNextHopTableAggregator(const Arts & arts)
//.........................................................................
//  
//-------------------------------------------------------------------------
ArtsNextHopTableAggregator::ArtsNextHopTableAggregator(const Arts & arts)
{
  assert(arts.Header().Identifier() == artsC_OBJECT_NEXT_HOP);
  
  this->_header = arts.Header();

  vector<ArtsAttribute>::const_iterator    inAttribute;

  for (inAttribute = arts.Attributes().begin();
       inAttribute != arts.Attributes().end(); inAttribute++) {
    this->_attributes.push_back(*inAttribute);
  }

  counter_t  NextHopCounter;
  vector<ArtsNextHopTableEntry>::const_iterator  inNextHopEntry;


  for (inNextHopEntry = arts.NextHopTableData()->NextHopEntries().begin();
       inNextHopEntry != arts.NextHopTableData()->NextHopEntries().end();
       inNextHopEntry++) {
    NextHopCounter.Pkts = inNextHopEntry->Pkts();
    NextHopCounter.Bytes = inNextHopEntry->Bytes();
    this->_NextHopCounters[inNextHopEntry->IpAddr()] =
      NextHopCounter;
  }
}

//-------------------------------------------------------------------------
//           void ArtsNextHopTableAggregator::Add(const Arts & arts)          
//.........................................................................
//  
//-------------------------------------------------------------------------
void ArtsNextHopTableAggregator::Add(const Arts & arts)
{
  assert(arts.Header().Identifier() == artsC_OBJECT_NEXT_HOP);

  vector<ArtsAttribute>::iterator          myHostAttribute;

  for (myHostAttribute = this->_attributes.begin();
         myHostAttribute != this->_attributes.end();
         myHostAttribute++) {
    if (myHostAttribute->Identifier() == artsC_ATTR_HOST)
      break;
  }
  
  vector<ArtsAttribute>::iterator          myPeriodAttribute;
  vector<ArtsAttribute>::const_iterator    inPeriodAttribute;
  
  for (myPeriodAttribute = this->_attributes.begin();
         myPeriodAttribute != this->_attributes.end();
         myPeriodAttribute++) {
    if (myPeriodAttribute->Identifier() == artsC_ATTR_PERIOD)
      break;
  }
  
  inPeriodAttribute = arts.FindPeriodAttribute();

  const uint32_t *myPeriod = myPeriodAttribute->Period();
  const uint32_t *inPeriod = inPeriodAttribute->Period();
  
  if (inPeriod[0] < myPeriod[0]) {
    myPeriodAttribute->Period(inPeriod[0],myPeriod[1]);
  }
  if (inPeriod[1] > myPeriod[1]) {
    myPeriodAttribute->Period(myPeriod[0],inPeriod[1]);
  }
    
  counter_t  NextHopCounter;
  vector<ArtsNextHopTableEntry>::const_iterator           inNextHopEntry;
  map<ipv4addr_t,counter_t,less<ipv4addr_t> >::iterator   myNextHopEntry;
  
  for (inNextHopEntry = arts.NextHopTableData()->NextHopEntries().begin();
       inNextHopEntry != arts.NextHopTableData()->NextHopEntries().end();
       inNextHopEntry++) {
    myNextHopEntry =
      this->_NextHopCounters.find(inNextHopEntry->IpAddr());
    if (myNextHopEntry == this->_NextHopCounters.end()) {
      NextHopCounter.Pkts = inNextHopEntry->Pkts();
      NextHopCounter.Bytes = inNextHopEntry->Bytes();
      this->_NextHopCounters[inNextHopEntry->IpAddr()] =
        NextHopCounter;
    }
    else {
      (*myNextHopEntry).second.Pkts += inNextHopEntry->Pkts();
      (*myNextHopEntry).second.Bytes += inNextHopEntry->Bytes();
    }
  }
  return;
}

//-------------------------------------------------------------------------
//      ArtsNextHopTable *                           
//      ArtsNextHopTableAggregator::ConvertToArtsNextHopTable() const 
//.........................................................................
//  
//-------------------------------------------------------------------------
ArtsNextHopTable *
ArtsNextHopTableAggregator::ConvertToArtsNextHopTable() const
{
  ArtsNextHopTableEntry  NextHopEntry;
  map<ipv4addr_t,counter_t,less<ipv4addr_t> >::const_iterator  NextHopCounter;
  ArtsNextHopTable  *artsNextHopTable = new ArtsNextHopTable();

  artsNextHopTable->Header() = this->_header;
  vector<ArtsAttribute>::const_iterator   inAttribute;
  
  for (inAttribute = this->_attributes.begin();
       inAttribute != this->_attributes.end(); inAttribute++) {
    artsNextHopTable->Attributes().push_back(*inAttribute);
  }

  uint64_t  totalPkts = 0;
  uint64_t  totalBytes = 0;
  
  for (NextHopCounter = this->_NextHopCounters.begin();
       NextHopCounter != this->_NextHopCounters.end(); NextHopCounter++) {
    NextHopEntry.IpAddr((*NextHopCounter).first);
    NextHopEntry.Pkts((*NextHopCounter).second.Pkts);
    NextHopEntry.Bytes((*NextHopCounter).second.Bytes);
    artsNextHopTable->NextHopEntries().push_back(NextHopEntry);
    totalPkts += NextHopEntry.Pkts();
    totalBytes += NextHopEntry.Bytes();
  }
  artsNextHopTable->TotalPkts(totalPkts);
  artsNextHopTable->TotalBytes(totalBytes);
  
  return(artsNextHopTable);
}
