//===========================================================================
//  @(#) $Name: arts++-1-1-a13 $
//  @(#) $Id: ArtsPortMatrixAggregatorMap.cc,v 1.2 2004/04/21 23:51:34 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include "artslocal.h"
}

#include <string>
#include <iterator>
#ifdef HAVE_FSTREAM
  #include <fstream>
#else
  #include <fstream.h>
#endif

#include "ArtsPortMatrixAggregatorMap.hh"

using namespace std;

static const std::string rcsid = "@(#) $Name: arts++-1-1-a13 $ $Id: ArtsPortMatrixAggregatorMap.cc,v 1.2 2004/04/21 23:51:34 kkeys Exp $";

//-------------------------------------------------------------------------
//          int ArtsPortMatrixAggregatorMap::Add(const Arts & arts)         
//.........................................................................
//  
//-------------------------------------------------------------------------
int ArtsPortMatrixAggregatorMap::Add(const Arts & arts)
{
  vector<ArtsAttribute>::const_iterator   hostAttribute;
  vector<ArtsAttribute>::const_iterator   ifIndexAttribute;
  ArtsAggregatorMapKey                    portmKey;

  if (arts.Header().Identifier() != artsC_OBJECT_PORT_MATRIX)
    return(-1);
  
  hostAttribute = arts.FindHostAttribute();
  if (hostAttribute != arts.Attributes().end()) {
    portmKey.Router(hostAttribute->Host());
  }
  else {
    portmKey.Router(0);
  }

  ifIndexAttribute = arts.FindIfIndexAttribute();
  if (ifIndexAttribute != arts.Attributes().end()) {
    portmKey.IfIndex(ifIndexAttribute->IfIndex());
  }
  else {
    portmKey.IfIndex(0);
  }

  if ((*this).find(portmKey) == (*this).end()) {
    (*this)[portmKey] = new ArtsPortMatrixAggregator(arts);
  }
  else {
    ((*this)[portmKey])->Add(arts);
  }
  
  return(0);
}

//-------------------------------------------------------------------------
//  int
//  ArtsPortMatrixAggregatorMap::AddAllFromFile(const string & fileName, 
//                                              bool quiet)              
//.........................................................................
//  
//-------------------------------------------------------------------------
int ArtsPortMatrixAggregatorMap::AddAllFromFile(const string & fileName,
                                                bool quiet)
{
  ifstream    artsStream(fileName.c_str());
  int         numAdded = 0;
  
  if (!artsStream) {
    return(-1);
  }

  istream_iterator<ArtsPortMatrix>  artsStreamIter(artsStream);
  istream_iterator<ArtsPortMatrix>  artsStreamEnd;

  for ( ; artsStreamIter != artsStreamEnd; artsStreamIter++) {
    //  Add the data to the map.
    if (this->Add(*artsStreamIter) == 0)
      numAdded++;
    if (!quiet) {
      cout << ".";
      cout.flush();
    }
  }
  artsStream.close();
  return(numAdded);
}
