//===========================================================================
//  $Name: arts++-1-1-a13 $
//  $Id: ArtsPortMatrixData.cc,v 1.2 2004/04/21 23:51:34 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================
#include <string>

#include "ArtsPortMatrixData.hh"
#include "ArtsPrimitive.hh"

using namespace std;

static const std::string rcsid = "@(#) $Name: arts++-1-1-a13 $ $Id: ArtsPortMatrixData.cc,v 1.2 2004/04/21 23:51:34 kkeys Exp $";


//----------------------------------------------------------------------------
//                 ArtsPortMatrixData::ArtsPortMatrixData()                
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsPortMatrixData::ArtsPortMatrixData()  
{
  this->_sampleInterval = 1;
  this->_count = 0;
  this->_totpkts = 0;
  this->_totbytes = 0;
  this->_orphans = 0;

  #ifndef NDEBUG
    ++this->_numObjects;
  #endif
}

//----------------------------------------------------------------------------
//                ArtsPortMatrixData::~ArtsPortMatrixData()                
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsPortMatrixData::~ArtsPortMatrixData()
{
  #ifndef NDEBUG
    --this->_numObjects;
  #endif
}

//-------------------------------------------------------------------------
//   istream & ArtsPortMatrixData::read(istream& is, uint8_t version)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
istream & ArtsPortMatrixData::read(istream& is, uint8_t version)
{
  uint32_t             entryNum;
  ArtsPortMatrixEntry  portEntry;
  
  g_ArtsLibInternal_Primitive.ReadUint16(is,this->_sampleInterval,2);
  g_ArtsLibInternal_Primitive.ReadUint32(is,this->_count,4);
  g_ArtsLibInternal_Primitive.ReadUint64(is,this->_totpkts,8);
  g_ArtsLibInternal_Primitive.ReadUint64(is,this->_totbytes,8);
  g_ArtsLibInternal_Primitive.ReadUint64(is,this->_orphans,8);
  
  this->_portEntries.reserve(this->_count);

  for (entryNum = 0; entryNum < this->_count; ++entryNum) {
    portEntry.read(is,version);
    this->_portEntries.push_back(portEntry);
  }
  return(is);
}

//-------------------------------------------------------------------------
//       int ArtsPortMatrixData::read(int fd, uint8_t version)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
int ArtsPortMatrixData::read(int fd, uint8_t version)
{
  uint32_t             entryNum;
  ArtsPortMatrixEntry  portEntry;
  int                  rc;
  int                  bytesRead = 0;
  
  rc = g_ArtsLibInternal_Primitive.ReadUint16(fd,this->_sampleInterval,2);
  if (rc < 2) {
    return(-1);
  }
  bytesRead += rc;
  
  rc = g_ArtsLibInternal_Primitive.ReadUint32(fd,this->_count,4);
  if (rc < 4) {
    return(-1);
  }
  bytesRead += rc;
  
  rc = g_ArtsLibInternal_Primitive.ReadUint64(fd,this->_totpkts,8);
  if (rc < 8) {
    return(-1);
  }
  bytesRead += rc;
  
  rc = g_ArtsLibInternal_Primitive.ReadUint64(fd,this->_totbytes,8);
  if (rc < 8) {
    return(-1);
  }
  bytesRead += rc;
  
  rc = g_ArtsLibInternal_Primitive.ReadUint64(fd,this->_orphans,8);
  if (rc < 8) {
    return(-1);
  }
  bytesRead += rc;
  
  this->_portEntries.reserve(this->_count);

  for (entryNum = 0; entryNum < this->_count; ++entryNum) {
    if ((rc = portEntry.read(fd,version)) < 0) {
      return(-1);
    }
    bytesRead += rc;
    this->_portEntries.push_back(portEntry);
  }
  return(bytesRead);
}


//-------------------------------------------------------------------------
//  ostream & ArtsPortMatrixData::write(ostream& os, uint8_t version)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ostream & ArtsPortMatrixData::write(ostream& os, uint8_t version)
{
  uint32_t       entryNum;

  g_ArtsLibInternal_Primitive.WriteUint16(os,this->_sampleInterval,2);
  this->_count = this->_portEntries.size();
  g_ArtsLibInternal_Primitive.WriteUint32(os,this->_count,4);
  g_ArtsLibInternal_Primitive.WriteUint64(os,this->_totpkts,8);
  g_ArtsLibInternal_Primitive.WriteUint64(os,this->_totbytes,8);
  g_ArtsLibInternal_Primitive.WriteUint64(os,this->_orphans,8);
  
  for (entryNum = 0; entryNum < this->_count; ++entryNum) {
    this->_portEntries[entryNum].write(os);
  }
  return(os);
}

//-------------------------------------------------------------------------
//       int ArtsPortMatrixData::write(int fd, uint8_t version)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
int ArtsPortMatrixData::write(int fd, uint8_t version)
{
  uint32_t       entryNum;
  int            rc;
  int            bytesWritten = 0;
  
  rc = g_ArtsLibInternal_Primitive.WriteUint16(fd,this->_sampleInterval,2);
  if (rc < 2) {
    return(-1);
  }
  bytesWritten += rc;

  this->_count = this->_portEntries.size();

  rc = g_ArtsLibInternal_Primitive.WriteUint32(fd,this->_count,4);
  if (rc < 4) {
    return(-1);
  }
  bytesWritten += rc;
  
  rc = g_ArtsLibInternal_Primitive.WriteUint64(fd,this->_totpkts,8);
  if (rc < 8) {
    return(-1);
  }
  bytesWritten += rc;
  
  rc = g_ArtsLibInternal_Primitive.WriteUint64(fd,this->_totbytes,8);
  if (rc < 8) {
    return(-1);
  }
  bytesWritten += rc;
  
  rc = g_ArtsLibInternal_Primitive.WriteUint64(fd,this->_orphans,8);
  if (rc < 8) {
    return(-1);
    bytesWritten += rc;
  }
  
  for (entryNum = 0; entryNum < this->_count; ++entryNum) {
    if ((rc = this->_portEntries[entryNum].write(fd,version)) < 0) {
      return(-1);
    }
    bytesWritten += rc;
  }
  return(bytesWritten);
}

//-------------------------------------------------------------------------
//     uint32_t ArtsPortMatrixData::Length(uint8_t version) const
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint32_t ArtsPortMatrixData::Length(uint8_t version) const
{
  uint32_t  length = 0;

  length += (sizeof(this->_sampleInterval) + sizeof(this->_count) +
             sizeof(this->_totpkts) + sizeof(this->_totbytes) +
             sizeof(this->_orphans));

  vector<ArtsPortMatrixEntry>::const_iterator  portEntryIter;
  for (portEntryIter = this->PortEntries().begin();
       portEntryIter != this->PortEntries().end(); ++portEntryIter) {
    length += portEntryIter->Length(version);
  }
  return(length);
}

//-------------------------------------------------------------------------
//  ostream& operator << (ostream& os,                   
//                        const ArtsPortMatrixData & artsPortMatrixData)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ostream& operator << (ostream& os,
                      const ArtsPortMatrixData & artsPortMatrixData)
{
  os << "PORTMATRIX OBJECT DATA" << endl;

  os << "\tsample_interval: " << artsPortMatrixData.SampleInterval()
     << endl;
  os << "\tcount: " << artsPortMatrixData.Count() << endl;
  os << "\ttotpkts: " << artsPortMatrixData.TotalPkts() << endl;
  os << "\ttotbytes: " << artsPortMatrixData.TotalBytes() << endl;
  os << "\torphans: " << artsPortMatrixData.Orphans() << endl;
  
  vector<ArtsPortMatrixEntry>::const_iterator  portEntryIter;

  for (portEntryIter = artsPortMatrixData.PortEntries().begin();
       portEntryIter != artsPortMatrixData.PortEntries().end();
       ++portEntryIter) {
    os << (*portEntryIter);
  }
  return(os);
}

//-------------------------------------------------------------------------
//               void ArtsPortMatrixData::SortEntriesByBytes()              
//.........................................................................
//  
//-------------------------------------------------------------------------
void ArtsPortMatrixData::SortEntriesByBytes()
{
  sort(this->_portEntries.begin(),this->_portEntries.end(),
       ArtsPortMatrixEntryGreaterBytes());
  return;
}

//-------------------------------------------------------------------------
//               void ArtsPortMatrixData::SortEntriesByPkts()               
//.........................................................................
//  
//-------------------------------------------------------------------------
void ArtsPortMatrixData::SortEntriesByPkts()
{
  sort(this->_portEntries.begin(),this->_portEntries.end(),
       ArtsPortMatrixEntryGreaterPkts());
  return;
}

uint32_t  ArtsPortMatrixData::_numObjects = 0;
